# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator/create_python_api.py script.
"""Public API for tf.__internal__.saved_model.load namespace.
"""

import sys as _sys

from tensorflow.python.saved_model.revived_types import VersionedTypeRegistration
from tensorflow.python.saved_model.revived_types import get_setter
from tensorflow.python.saved_model.revived_types import register_revived_type
from tensorflow.python.saved_model.revived_types import registered_identifiers